;================================
;          ZX-Assembler-2 
;      by Artic Computing LTD
;================================
; A compilation of video routines
; GCH 2021
;================================

ORG 4D00			;19712

@Keyboard=02BB			;Keyboard scanning subroutine
@Decode=07BD			;Keyboard decode subroutine
@PrntAt=08F5			;PRINT AT Y,X ROM routine
@DFile=400C			;Address of display file
@Vars=4010			;Start of Vars
@LastK=4025			;Keyboard debounce value
@Frames=4034			;16436, Frame counter
@Coords=4036			;16438, PLOT coords
@Pattern=403C			;16444, Pattern store
@Cell=403C			;16444, Cell character
@BKG=403D			;16445, Background character
@Density=403E			;16446, Density value
@DanceDat=4048			;16461, Dancers data
@LoopFlag=4083			;16515, Player On/OFF flag
@PrntPos=49AF			;18863,	Calculate Print Position, a "PLAYER" routine
;================================
@WorkArea=49CB			;Address of WorkArea
@altDFile=49CB			;Address of altDFile
@WorkDF=4043
;================================
; Jump Table
;================================
	JP BallRand		;19712 A-00 BallLIFE random loop
	JP BallSym		;19715 B-03 BallLIFE 4 fold symm loop
	JP GoodRnd		;19718 G-06 GoodLIFE random loop
	JP GoodSym		;19721 H-09 GoodLIFE 4 fold symm loop
	JP DiceLoop		;19724 D-0C Dice loop
	JP Labyrinth		;19727 L-0F Endless maze
	JP Scope		;19730 K-12 Kaleidoscope
	JP PxlMandala		;19733 P-15 Pixel Mandala
	JP RandLoop		;19736 R-18 Random Screen
	JP SymA			;19739 T-1B 4 Fold Sym Style A
	JP SymB			;19742 U-1E 4 Fold Sym Style B
	JR WallPaper		;19745 W-21 Wallpaper
	JP ZXMandala		;19747 Z-23 ZX81 Mandala
	JP Vertigo		;19750 V-26 Vertigo
	JP Survival		;19753 S-29 Survival
	JP Cityscape		;19756 C-2C Cityscape
	JP Dancers		;19759 E-2F Dancers
;================================
; Wallpaper Loop
;================================
@WallPaper
	CALL SetUpDF
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	AND 01
	LD C,A
	JR Z +2			;0, move on
	LD C,80			;1, C=128 for graphic chrs
	LD HL,Pattern		;Pattern data address
	LD B,07			;Byte count
	PUSH HL			;PUSH Pattern address
	LD E,0A			;Set upper limit
	CALL Modulus		;A=0 to 9
	POP HL			;POP Pattern address
	ADD A,C			;Add 0 or 128 to A
	LD (HL),A		;Store chr code
	INC HL			;Increase pattern addr
	DJNZ -12		;Loop until B=0
	XOR A			;Initialize counter
	LD DE,(WorkDF)		;DE=work area
	INC DE			;Skip 1st N/L chr
	LD HL,Pattern		;Source
	LD C,07			;Byte count
	LDIR			;Copy pattern to altDFile 
	INC A			;Increase counter
	CP 71			;Upper limit
	JR NZ -12		;Repeat 113 times
	LD B,18			;Restore N/L chrs
	LD HL,(WorkDF)		;Get work area
	LD DE,0021		;Row count
	ADD HL,DE		;Next row
	LD (HL),76		;Set N/L
	DJNZ -5			;Loop until B=0
	CALL SwapDF
	LD A,(DFile+1)		;Test which DFile is being displayed
	CP 59			;altDFile?
	JR NZ +5		;No, move on
	CALL Delay		;Yes, skip Keysort call
	JR WallPaper+3
	CALL KeySort		;Loop until "Q"uit pressed
	JR WallPaper+3
;================================
; Four fold symmetry, style A
;================================
@SymA
	CALL SetDensity		;Set density range to 87 to 150
	CALL CLSBW		;CLS with " " or "g "
	CALL PikCell		;Pick a chr to print
;Symloop
	CALL StyleA		;Run style A 4 fold pattern
	CALL KeySort		;Get key press
	JR -8			;No, loop 
;================================
@StyleA
	LD E,0C			;Set upper limit, 12
	CALL Modulus
	LD B,A          	;B=Ycoord
	LD E,10			;Set upper limit, 16
	CALL Modulus
        LD C,A          	;C=Xcoord
	CALL PrintCell		;Get chr to print, (BKG) or (Cell)
	JP PrintQuads		;Print chr to the four quadrants
;================================
; Four fold symmetry, style B
;================================
@SymB
	CALL SetDensity		;Set density range to 87 to 150
	CALL CLSBW		;CLS with " " or "g "
	CALL RND8Bit
	AND 01			;Odd or Even?
	LD A,08
	JR Z +5
	LD A,(BKG)
	ADD A,80
	LD (BKG),A		;Set clear chr 00,08 or 80
	CALL PikCell		;Pick a chr to print
@SymBloop
	CALL PrintCell		;Get chr to print, (BKG) or (Cell)
	PUSH AF			;PUSH chr to print
	LD E,10			;Set upper limit, 16
	CALL Modulus
	LD B,A			;Set Ycoord
	CALL Modulus
	LD C,A			;Set Xcoord
	POP AF			;POP chr to print
	PUSH BC
	CALL PrintQuads		;Print to the 4 quadrants
	POP BC			;POP Y/Xcoords
	LD A,B			;Swap Y/Xcoords
	LD B,C
	LD C,A
	CALL PrintQuads		;Print to the 4 quadrants
	CALL KeySort		;Test keys
        JR SymBloop
;================================
; LIFE random screen
;================================
@LIFEScr
	LD HL,(DFile)		;Get work area
	INC HL			;Skip 1st N/L chr
	LD BC,0318		;Byte count
	PUSH HL			;PUSH DFile address
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	LD L,A			;L=range 0 to 255
	LD A,(Density)		;Get density value
	CP L			;L>Density?
	LD A,(Cell)		;"gO" or GOODLife cell
	JR C +2			;Yes, Print cell
	LD A,80			;"g "
	POP HL			;POP DFile address
	BIT 6,(HL)		;N/L?
	JR NZ +1		;Yes, skip print
	LD (HL),A		;Print cell or bkg
	INC HL			;Next DFile address
	DEC BC 			;Decrease counter
	LD A,B 
	OR C 
	JR NZ LIFEScr+7		;Loop until BC=0
	RET
;================================
; GoodLIFE random screen loop
;================================
@GoodRnd
	CALL GoodPars		;Pick a cell chr to print from list
	CALL LIFEScr		;Print random screen
	CALL GoodLIFE		;Run one gen
	CALL KeySort		;Get key press
	DEC (IY+09)		;Decrease gen counter
	JR NZ GoodRnd+6		;0? No, run another gen
	JR GoodRnd		;Loop until "Q"uit pressed
;================================
; GoodLIFE 4 fold symm loop
;================================
@GoodSym
	CALL GoodPars
	CALL StyleA		;Run 4 fold symm style A
	DEC (IY+09)		;Decrease gen counter
	JR NZ GoodSym+3		;Loop until counter=0
	LD (IY+09),3C		;Set gen counter to 60
	CALL GoodLIFE		;Run one gen
	CALL KeySort		;Get key press
	DEC (IY+09)		;Decrease gen counter
	JR NZ -11		;0? No, run another gen
	JR GoodSym		;Yes, start a new pattern
;================================
; Set GoodLIFE parameters
;================================
@GoodPars
	CALL SetDensity		;Set density range to 87 to 150
	CALL CLS-2		;CLS with "g "
	CALL PikCell		;Pick a cell chr to print from list
	RES 7,A			;Graphic chrs not allowed
	AND A			;0 not allowed
	JR Z GoodPars+6
	LD (Cell),A		;Set cell chr
	LD (Cell2+1),A		;Set cell chr
	LD (Cell3+1),A		;Set cell chr
	LD B,A			;Save cell chr to B
	CALL RND8Bit
	AND 01
	JR Z +4			;Jump if even
	LD A,88			;Use upper grey block as birth marker
	JR +3
	LD A,B			;Restore cell chr from B
	ADD A,80		;Birth chr=Cell chr+128
	LD (Baby+1),A		;Set birth chr
	LD (Cell3+2),A		;Set birth chr
	LD (IY+09),FF		;Set gen counter to 255
	RET
;================================
; The ZX81 GAME of LIFE
; Wilf Rigter @ 1983?
; Modified by GCH/2021
;================================
@GoodLIFE
	LD HL,(DFile) 		;HL points to the top of the DFile
	LD DE,0021		;Bytes/row
	LD C,16			;22 rows
@NxtLine
	LD B,20			;32 cells per line
@Analyze
	LD A,50			;Initialize cell counter in A with 50h
	CP (HL)			;This part checks block of 9 neighbouring cells
	ADC A,D			;increments ">80h" neighbour counter in A
	INC HL		 	;NCell
	CP (HL)
	ADC A,D
	INC HL		 	;NECell
	CP (HL)
	ADC A,D
	ADD HL,DE		;ECell
	CP (HL)
	ADC A,D
	ADD HL,DE		;SECell
	CP (HL)
	ADC A,D
	DEC HL			;SCell
	CP (HL)
	ADC A,D
	DEC HL			;SWCell
	CP (HL)
	ADC A,D
	SBC HL,DE		;WCell
	CP (HL)
	ADC A,D
	INC HL			;Home cell
	CP (HL)
	JR NC +6		;If home cell is >56
	CP 55
	JR NZ +12
@Baby
	LD (HL),00		;Mark for birth
	CP 55
	JR Z +6
	CP 56
	JR Z +2
;Death
	LD (HL),08		;Mark as dying
;Done
	XOR A
	SBC HL,DE
	DJNZ Analyze
	INC HL		 	;Advance to the next line
	DEC C			;Dec line counter
	JR NZ NxtLine		;Last line?
				;Now DFile is Tested for dying
				;and birth cell marks
				;first execute dying cells
	LD DE,0880		;D=Dying, E=Dead (BKG)
@Mark
	LD HL,(DFile)		;Get DFile
	LD BC,03F6		;Cell Counter, 3*246
@NMark
	INC HL			;First cell
	LD A,(HL)		;Test cell
	CP D	   		;Is it a dying cell (CHR$ 08)?
	JR NZ +1  		;Skip if not dying
	LD (HL),E    		;Set as BKG if dying cell
;NoDie
	DEC C			;Decrease cell counter
	JR NZ NMark  		;Next cell
	DEC B	  		;Next cell
	JR NZ NMark  		;Next cell
				;The same routine is used for birth
@Cell2
	LD A,00			;now execute the birth cycle
	CP E	   		;did we already do the births?
	JR Z +5     		;If birth and death done then bye
@Cell3
	LD DE,8800   		;D=Birth, E=Cell
	JR Mark			;Go forth and deliver
;End
	LD C,21			;Byte count
	SBC HL,BC
	INC HL			;HL=Row 23 addr-1 (N/L chr of row 22)
	LD DE,(DFile)		;DE=Row 01 addr-1 (N/L start of DFile)
	LDIR			;Copy row 23 to row 01
	EX DE,HL		;Copy row 02 to row 24
				;HL=Row 02 addr-1 (N/L chr of row 01)
				;DE=Row 24 addr-1 (N/L chr of row 23)
	LD C,21			;Byte count
	LDIR
	RET
;================================
; Ball LIFE random screen loop
;================================
@BallRand
	CALL SetDensity		;Set density range to 87 to 150
	LD (IY+09),FF		;Set Generations counter
	LD (IY+60),B4		;Cell chr "gO"
	CALL LIFEScr
	CALL BallLIFE		;Run one gen
	CALL KeySort		;Get key press
	DEC (IY+09)		;Decrease generations counter
	JR NZ -11		;0? No, run another Gen
	JR BallRand		;Loop until "Q"uit pressed
;================================
; Ball LIFE symm screen loop
;================================
@BallSym
	CALL SetDensity		;Set density range to 87 to 150
	LD (IY+60),B4		;"gO"
	CALL CLS-2		;CLS with "g "
	LD (IY+09),FF		;Set symm gen counter to 255
	CALL StyleA		;Run 4 fold symm style A
	DEC (IY+09)		;Decrease generations counter
	JR NZ -8		;0? No, run another Gen
	LD (IY+09),3C		;Set LIFE gen counter to 60
	CALL BallLIFE		;Run one gen
	CALL KeySort		;Get key press
	DEC (IY+09)		;Decrease gen counter
	JR NZ -11		;0? No, run another gen
	JR -29			;Yes, start a new pattern
;==============================
; Ball LIFE
; John Ball
;==============================
@BallLife
	LD HL,(WorkArea)	;Copy DFile to WorkArea
	LD BC,0021 		;Row size+N/L
	ADD HL,BC 		;2nd Row
	EX DE,HL 		;Destination=WorkArea+33
	LD HL,(DFile)		;Source=DFile
	LD BC,0319 		;Byte count
	PUSH BC			;PUSH byte counter
	PUSH HL  		;PUSH DFile
	PUSH DE 		;PUSH Row 1 addr
	LDIR 			;Transfer DFile to WorkArea
	POP HL 			;POP Row 1 addr
	PUSH HL 		;PUSH Row 1 addr
	LD B,18 		;Row counter
@ScanStor
	INC HL 			;Next byte in row
	BIT 6,(HL) 		;N/L?
	JR NZ +7		;Yes, move on
	BIT 5,(HL) 		;"O"?
	CALL NZ Analyse		;Yes, analyze
	JR ScanStor		;Loop until N/L found
	DJNZ ScanStor		;Loop until B=0
;Update row 24 cell values
	POP DE 			;POP Row 1 addr
	PUSH DE			;PUSH Row 1 addr
	LD HL,FFDF 		;-32
	ADD HL,DE 		;
	EX DE,HL 		;
	LD HL,0318 		;Offset to row 24
	ADD HL,DE 		;HL=Row 24, DE=Row 0
	CALL UpDate		;Update row 24 with row 0 values
	POP DE 			;POP Row 1 addr
	PUSH DE			;PUSH Row 1 addr
;Update row 1 cell values
	LD HL,0318 		;Offset to row 25
	ADD HL,DE 		;HL=Row 25
	EX DE,HL 		;HL=Row 1 addr, DE=Row 25
	CALL UpDate		;Update row 1 with row 25 values
;Plot cells based on values stored in Workarea
	POP HL 			;POP Row 1 addr
	PUSH HL			;POP Row 1 addr
	INC HL 
	LD B,18			;Row count 
@NxtCell
	LD A,(HL)		;Get cell value 
	AND FE 			;
	XOR B6 			;Cell+2 (36/B6)
	JR NZ Live
@Birth
	LD (HL),B4		;Cell   (34/B4)
	JR NxtChr
@Live
	LD A,(HL) 
	XOR 83 			;BKG+3  (03/83)
	JR Z Birth
@Death
	LD (HL),80		;BKG    (00/80)
@NxtChr
	INC HL 			;Process entire workarea
	LD A,76 
	CP (HL) 
	JR NZ NxtCell
	INC HL 
	DJNZ NxtCell
	POP HL 			;POP Workarea+33
	POP DE 			;POP DFile
	POP BC 			;POP count
	LDIR 			;Copy Workarea to DFile
	RET			;Done
;=============================
;Calculate row 1 & row 24 cell values
;=============================	
@UpDate
	LD B,20			;32 Cells/row
	INC HL
	INC DE
	LD A,(DE)
	ADD A,(HL)
	ADD A,80		;BKG (00/80)
	LD (HL),A
	LD A,80			;BKG (00/80)
	LD (DE),A
	DJNZ Update+2
	RET
;==============================
;Analyze current cell's conditions
;==============================
@Analyse
	PUSH BC 
	PUSH HL 
	LD DE,0021 	;Row length + N/L
	AND A 		;Clear C
	SBC HL,DE 	;N cell
	LD A,76 	;N/L char.
	LD BC,0000 	;Row offset 0000
	INC (HL) 	;N cell value
	DEC HL 		;NW cell
	CP (HL) 	;Is it a N/L?
	JR NZ +3
	LD C,20 	;Yes
	ADD HL,BC 	;Wrap right
	INC (HL) 	;NW cell value
	ADD HL,DE 	;W cell
	INC (HL) 	;W cell value
	ADD HL,DE 	;SW cell
	INC (HL) 	;SW cell value
	AND A
	SBC HL,BC 	;BC=00 or 20 if wrap right 
	LD C,B	 	;Row offset 0000
	INC HL 		;S cell
	INC (HL)	;S cell value
	INC HL 		;SE cell
	CP (HL) 	;Is it a N/L?
	JR NZ +5
	LD C,20 	;Yes
	AND A
	SBC HL,BC 	;Wrap left
	AND A
	INC (HL)	;SE cell value
	SBC HL,DE 	;E cell
	INC (HL)	;E cell value
	SBC HL,DE 	;NE cell
	INC (HL)	;NE cell value
	POP HL 
	POP BC 
	RET 
;================================
; ZX81 Mandala
;================================
@ZXMandala
	CALL CLSBW		;CLS with " " or "g "
	LD A,(BKG)		;Get BKG value
	LD (MandSet+1),A	;Set BKG value
	CALL PikCell		;Get cell chr
	LD (MandSet+5),A	;Set cell chr
	CALL Rnd8Bit		;Get random number
	AND 01
	JR NZ +4		;1, print cell chr
@MandSet
	LD A,00			;0, print BKG chr
	JR +2
	LD A,00			;Print cell chr
	LD (Setchr+1),A		;Set chr to print
	LD E,15			;Set upper limit
	CALL Modulus
	LD B,A			;B=0 to 21
	LD (Coords+1),A		;Save coord
	CALL Modulus
	LD C,A			;C=0 to 21
	LD (Coords),A		;Save coord
	LD A,08
	ADD A,C
	LD C,A
	CALL Print		;B,C+8
	LD E,C
	LD A,08
	ADD A,B
	LD C,A
	LD B,E
	CALL Print		;C,B+8
	LD E,C
	LD A,18
	SUB B
	LD C,A
	LD B,E
	CALL Print		;C,24-B
	LD A,18
	SUB C
	LD C,A
	CALL Print		;B,24-C
	LD A,17
	SUB B
	LD B,A
	LD A,08
	ADD A,C
	LD C,A
	CALL Print		;23-B,C+8
	LD E,B
	LD A,17
	SUB C
	LD B,A
	LD A,08
	ADD A,E
	LD C,A
	CALL Print		;23-C,B+8
	LD E,B
	LD A,17
	SUB C
	LD B,A
	LD A,18
	SUB E
	LD C,A
	CALL Print		;23-C,24-B
	LD A,17
	SUB B
	LD B,A
	LD A,18
	SUB C
	LD C,A
	CALL Print		;23-B,24-C
	CALL KeySort		;Check keys
	JR MandSet-7		;Loop until "Q" pressed
;================================
; Print chr
;================================
@Print
	CALL PrntPos		;Calculate print position
@Setchr
	LD (HL),00		;Print " " or "g "
	LD BC,(Coords)		;Restore coords
	RET
;================================
; Dice Loop
;================================
@DiceLoop
	CALL RND8Bit
	AND 7F 
	LD (IY+09),A		;Set Generations counter
	AND 07			;A=0 to 7
;	CALL Roll
	LD B,00			;Set BC
	LD C,A			;A=1 to 6
	LD HL,Felt		;Get dice table background chr
	ADD HL,BC		;Set pointer
	LD A,(HL)		;Get bkg chr
	LD (BKG),A		;Set bkg chr
	CALL CLS		;CLS with felt chr
	CALL DicePrnt		;Print dice
	CALL Throw		;Throw dice
	CALL KeySort		;Check keys
	DEC (IY+09)		;Decrease Generations counter
	JR NZ -14		;If not 0 keep rolling the dice
	LD BC,FF01	 	;Pause the display to read the roll
	DEC BC 
	LD A,B 
	OR C 
	JR NZ -5
	JR DiceLoop		;Reroll the dice
;================================
; Print dice
;================================
@DicePrnt
	LD HL,(DFile)		;Get DFile
	LD DE,00C9		;Offset to left die print position
	ADD HL,DE		;Left die print p[osition
	PUSH HL			;PUSH left die print position
	CALL Die		;Print left die
	POP HL			;POP left die print position
	LD E,0F			;Offset to right die position
	ADD HL,DE		;Right die print position
@Die
	LD DE,Line1		;Line 1 data
	CALL DiePrnt		;Print line 1
	LD B,0A			;Loop counter
	PUSH BC			;PUSH loop counter
	LD DE,Line2		;Lines 2 to 11 data
	CALL DiePrnt		;Print lines 2 to 11
	POP BC			;POP loop counter
	DJNZ -10		;Loop until B=0
	LD DE,Line12		;Line 12 data
@DiePrnt
	PUSH HL			;PUSH print position
	LD A,(DE)		;Get chr to print
	LD (HL),A		;Print chr
	INC DE			;Next chr
	LD A,(DE)		;Get chr to print
	LD B,0A			;Loop counter
	INC HL			;Next print position
	LD (HL),A		;Print chr
	DJNZ -4			;Loop until B=0
	INC HL			;Next print position
	INC DE			;Next chr
	LD A,(DE)		;Get chr to print
	LD (HL),A		;Print chr
	POP HL			;POP print position
	LD DE,0021		;Offset to next row
	ADD HL,DE		;Set print position to next row
	RET
;================================
@Throw
	CALL Roll		;Roll left die
	LD BC,0021		;Bytes/row
	LD HL,(DFile)		;Get DFile
	INC HL
	PUSH HL			;PUSH DFile
	CALL Pips
	CALL Roll		;Roll right die
	POP HL			;POP DFile		
	LD DE,000F		;Offset to right die
	ADD HL,DE
@Pips
	BIT 0,A			;Odd or Even roll?
	JR Z +9			;Jump if even
	LD DE,0172		;Else print center Pip
	CALL PrintPip
	CP 01			;1?
	RET Z			;Yes, return
	LD DE,00EA		;NW Pip
	CALL PrintPip
	LD DE,01FA		;SE Pip
	CALL PrintPip
	CP 04			;2,3?
	RET C			;Yes, return
	LD DE,00F2		;NE Pip
	CALL PrintPip
	INC D			;SW Pip
	CALL PrintPip
	CP 06			;4,5?
	RET NZ			;Yes, return
	LD DE,016E		;W Pip
	CALL PrintPip
	LD E,76			;E Pip
@PrintPip
	PUSH HL			;PUSH (DFile)+1
	PUSH AF			;PUSH rnd number
	LD A,80			;Pip chr
	ADD HL,DE		;Set print position
	LD (HL),A
	INC HL
	LD (HL),A
	ADD HL,BC
	LD (HL),A
	DEC HL
	LD (HL),A
	POP AF			;POP rnd number
	POP HL 			;POP (DFile)+1
	RET
;================================
; Roll die
; Returns A=1 to 6
;================================
@Roll
	CALL RND8BIT
	AND 07			;Range 0 to 7
	JR Z Roll		;Retry if 0
	CP 07			;7?
	JR Z Roll		;Retry if 7
	RET
;================================
; Endless Maze
;================================
@Labyrinth
	CALL ScrollUp
	LD C,20
	SBC HL,BC		;Start of line 24
	PUSH HL			;PUSH print location
	CALL RND8Bit		;Get random number
	AND 0F			;Range 0 to 15
	LD HL,MazeChr		;Chr list
	LD C,A			;BC=Location of chr in list
	ADD HL,BC		;Point to chr addr
	LD A,(HL)		;Get chr
	POP HL			;POP print location
	LD (HL),A		;Print chr
	INC HL			;Next print location
	BIT 6,(HL)		;Test for N/L chr
	JR Z -19		;Print next chr
	CALL KeySort		;Loop until "Q"uit pressed
	JR Labyrinth		;No, loop until keypressed
;================================
; Kaleidoscope By S.H. McGurrin
; Syntax v5,n01 p.19
;================================
@Scope
	CALL Release		;Wait for no key press
	CALL CLSBW		;No, CLS with " " or "g "
@ScopeLup	
	LD E,3F			;Set upper limit
	CALL Modulus		;Get Xcoord, range=0 to 63
	LD C,A
	LD E,2F			;Set upper limit
	CALL Modulus		;Get Ycoord, range=0 to 47
	LD B,A			;BC=Y/Xcoords
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	AND 01
        JR Z +2			;Jump if even
        LD A,40            	;40 FOR Plot, 0 FOR UNPlot
	PUSH AF		   	;PUSH Plot/UnPlot Flag
	CALL Plot		;PLOT/UNPLOT X,Y
        LD A,2F			;47
        SUB B
        LD B,A
	POP AF 		   	;POP Plot/UnPlot Flag
	PUSH AF		   	;PUSH Plot/UnPlot Flag
	CALL Plot		;PLOT/UNPLOT X,47-Y
        LD A,3F			;63
        SUB C
        LD C,A
	POP AF		   	;POP Plot/UnPlot Flag
	PUSH AF		   	;PUSH Plot/UnPlot Flag
	CALL Plot		;PLOT/UNPLOT 63-X,47-Y
        LD A,2F			;47
        SUB B
        LD B,A
	POP AF		 	;POP Plot/UnPlot Flag
	CALL Plot		;PLOT/UNPLOT 63-X,Y
	CALL KeySort
        JR ScopeLup
;================================
; 64x48 Plot Routine
;
;  B=YCoord, 0 to 43
;  C=XCoord, 0 to 63
;  A=40H/64=PLOT
;  A=00=UNPLOT
;===============================
@Plot
	PUSH BC			;PUSH X,Y coords
	PUSH AF			;PUSH Plot/UnPlot flag
	LD A,87			;A=87=NW Pixel
	BIT 0,B			;Y coord
	JR NZ +2		;Odd line number?
	ADD A,10		;A=97=SW Pixel 
	BIT 0,C			;X coord
	JR Z +2			;Even col number?
	ADD A,08		;A=8F=NE Pixel or A=9F=SE Pixel
	LD D,A			;D=Pixel quadrant value
	POP AF			;POP Plot/UnPlot flag
	ADD A,D			;A=D IF UNPlot or D+64 IF Plot
	LD (PlotPX+1),A		;Self modifying code
	SRL B			;Convert Y-coord to Print coord, Y/2
	SRL C			;Convert X-coord to Print coord, X/2
	LD HL,(DFile)		;Get DFile
	INC HL			;Start of DFile
	LD D,00
	LD E,C			;DE=Column number
	ADD HL,DE		;HL=Column position in DFile
	LD A,17			;23-Y/2
	SUB B
	LD B,A			;B=Row number
	JR Z +6
	LD DE,0021		;Bytes/row
	ADD HL,DE
	DJNZ -3
	LD A,(HL)		;Get chr at DFile addr
	CP 08			;LO Pixel Chr
	JR C PlotPX
	CPL			;HI Pixel Chr
@PlotPX				
	SET 00,A		;00=SR SET/RES X,A
	BIT 3,A
	JR Z +5
	CPL
	AND 8F
	JR +2
	AND 0F			;Low value PIXEL, codes 1 to 7
	LD (HL),A		;Hi value PIXEL, codes 128 to 135
	POP BC			;POP X,Y coords
	RET
;================================
; Pixel Mandala
;================================
@PxlMandala
	CALL CLSBW		;CLS with "g " or " "
	BIT 7,D			;Test CLS chr
	JR Z Mandala		;Skip next section if D was 0
	LD BC,0020		;Print last line
	SBC HL,BC
	DEC C
	LD (HL),03		;Pixel line chr
	LD D,H
	LD E,L
	INC DE
	LDIR
@Mandala
	LD A,40			;Plot flag
	LD (MandPlot+1),A	;Set for plot
	LD B,02 
@MandLoop
	PUSH BC 		;PUSH counter
	LD E,2B			;Set upper limit to 45
	CALL Modulus		;Get Xcoord, range 0 to 45
	LD D,A			;Xcoord
	PUSH DE 		;PUSH Xcoord
	CALL Modulus		;Get Ycoord, range 0 to 45
	POP DE 			;POP Xcoord
	LD E,A 			;DE=XYcoords
	LD (Coords),DE		;Store coords
	CALL QuadPlot		;Plot quads
	XOR A			;Unplot flag
	LD (MandPlot+1),A	;Set for unplot
	POP BC 			;POP counter
	DJNZ MandLoop		;Loop back
	CALL KeySort
	JR Mandala
;================================
; Mandala Plot Quads
;  DE=Initial coords
;================================
@QuadPlot
	LD A,0C
	ADD A,D
	LD C,A
	LD B,E
	CALL MandPlot		;D+12,E
	LD A,0C
	ADD A,E
	LD C,A
	LD B,D
	CALL MandPlot		;E+12,D
	LD A,0C
	ADD A,E 
	LD C,A
	LD A,28 
	SUB D 
	LD B,A
	CALL MandPlot		;E+12,40-D
	LD A,0C 
	ADD A,D
	LD C,A
	LD A,28 
	SUB E 
	LD B,A
	CALL MandPlot		;D+12,40-E
	LD A,34 
	SUB D 
	LD C,A
	LD B,E
	CALL MandPlot		;52-D,E
	LD A,34 
	SUB E 
	LD C,A
	LD B,D
	CALL MandPlot		;52-E,D
	LD A,34 
	SUB E 
	LD C,A
	LD A,28 
	SUB D 			
	LD B,A
	CALL MandPlot		;52-E,40-D
	LD A,34 
	SUB D 
	LD C,A
	LD A,28 
	SUB E 
	LD B,A
	CALL MandPlot		;52-D,40-E
	RET
;================================
; PLOT Mandala Quads
;================================
@MandPlot
	LD A,00			;PLOT flag
	INC B			;Increase Ycoord before plotting
	INC B
	INC B
	INC B
	CALL Plot
	LD DE,(Coords)	 	;Restore coords
	RET
;================================
; Random screen loop
;================================
@RandLoop
	CALL SetUpDF
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	LD (Density),A		;Set density value
	AND 01
	JR Z +2			;0, move on
	LD A,80			;1, "g "
	LD (BKG),A		;Set to BKG=" " or "g "
	CALL PikCell		;Pick a chr to print
	CALL RandomScr		;Print random screen
	LD A,(DFile+1)		;Test which DFile is being displayed
	CP 59			;altDFile?
	JR NZ +5		;No, move on
	CALL Delay		;Yes, skip Keysort call
	JR -15
	CALL KeySort		;Loop until "Q"uit pressed
	JR -20
;================================
; Generate a 32x24 random screen
;================================
@RandomScr
	LD HL,(WorkDF)		;Get work area
	INC HL			;Skip 1st N/L chr
	LD BC,0318		;Byte count
	PUSH HL			;PUSH altDFile address
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	LD L,A			;L=range 0 to 255
	LD A,(Density)		;Get density value
	CP L			;L>Density?
	LD A,(Cell)		;Get cell
	JR C +3			;Yes, Print cell
	LD A,(BKG)		;No, Print bkg
	POP HL			;POP altDFile address
	BIT 6,(HL)		;N/L?
	JR NZ +1		;Yes, skip print
	LD (HL),A		;Print cell or bkg
	INC HL			;Next altDFile address
	DEC BC 			;Decrease counter
	LD A,B 
	OR C 
	JR NZ RandomScr+7	;Loop until BC=0
;================================
; This routine makes for smoother animation as it prevents the 
; "ripple effect" seen when updating the full screen display
;================================
@SwapDF
	LD A,(WorkDF+1)		;Which DFile?
	CP 59			;altDFile?
	JR NZ ZXDFile		;No, move on
	LD HL,(altDFile)	;Yes, display altDFile
	LD (DFile),HL
	LD HL,(ZXDFile+1)	;Use DFile as work area
	JR +9
@ZXDFile
	LD HL,0000		;Display DFile
	LD (DFile),HL
	LD HL,(altDFile)	;Use altDFile as work area
	LD (WorkDF),HL		;Set work area to altDFile or DFile
	RET
;================================
; Set density value range 87 to 150
;================================
@SetDensity
	CALL Release		;Wait for key release
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	AND 3F			;Range 0 to 63
	LD B,A			;B=0 to 63
	LD A,96			;A=150
	SUB B			;A=150-B, range 87 to 150
	LD (Density),A		;Set density value
	RET
;================================
; Survival
;================================
@Survival
	CALL SetUpDF
	LD BC,0000		;Use chrs 0 to 16, NOP NOP
	CALL RND8Bit		;Get random number
	BIT 00,A		;Even?
	JR Z +3			;0, move on
	LD BC,FFCB		;1, Use chr 128 to 144, (SET 7,A)
	LD (Settings+2),BC	;Set chr style
	AND 0F			
	INC A			;Range 1 to 16
	LD (Settings+1),A	;Set chr range
@Survivor
	INC D
	LD HL,(WorkDF)		;Switch between altDFile & DFile
	INC HL
	LD C,18			;Row count
@LoopY
	LD B,20			;Column count
@LoopX
	LD A,D
	AND 1F
	BIT 4,A
	JR Z +3
	CPL
	ADD A,20
	SUB C
	JR C +1
	CPL
	LD E,A
	LD A,D
	SRL A
	SRL A
	BIT 5,A
	JR Z +3
	CPL
	ADD A,40
	SUB B
	JR C +1
	CPL
	AND E
	ADD A,D
	SRL A
@Settings
	AND 00			;Bitwise chr filter, 1 to 16 from above
	NOP			;NOPs or SET 7,A
	NOP
	LD (HL),A
	INC HL
	DJNZ LoopX
	LD (HL),76
	INC HL 
	DEC C 
	JR NZ LoopY
	INC D
	CALL SwapDF
	LD A,(DFile+1)		;Test which DFile is being displayed
	CP 59			;altDFile?
	JR NZ +5		;No, move on
	CALL Delay		;Yes, skip Keysort call
	JR Survivor		;Loop
	LD A,D			;Save D, PUSH DE not allowed
	LD (PushD+1),A
	CALL KeySort
@PushD
	LD D,00			;Restore D
	JR Survivor
;================================
; Cityscape
;================================
@Cityscape
	CALL RND8Bit		;Get random number
	LD (Starz+1),A		;Set random number
	AND 0F			;Range 0 to 15
	LD B,A			;Ycoord, 0 to 15
	LD C,1F			;Xcoord, 31
	CALL PrntAt		;Set PRINT AT
	LD A,(HL)		;Get (HL) chr
	CP 80			;Building chr?
	JR NZ +17		;Skip star print if building chr
@Starz
	LD A,00			;Random number
	CP 20
	LD B,97			;Star chr "g*"
	JR C +8			;Print "g*" if 0 to 31
	CP 40
	LD B,95			;Star chr "g+"
	JR C +2			;Print "g+" if 32 to 63
	LD B,9B			;Star chr "g."
	LD (HL),B		;Otherwise print "g."
;ScrollLeft
	LD HL,(DFile)		;Get DFile
	INC HL			;Skip first N/L
	LD A,18			;Row counter
	LD B,00			;Clear B
	LD D,H			;Set destination
	LD E,L
	PUSH DE			;PUSH destination
	INC HL			;Set Source
	LD C,1F			;Byte count
	LDIR			;Shift row left
	POP HL			;POP destination
	LD C,21			;Bytes/row
	ADD HL,BC		;Next row
	DEC A			;Decrease row counter
	JR NZ -15		;Loop until A=0
;Build skyscraper
	CALL RND8Bit		;Get random number
	LD B,A			;Save random number
	AND 01			;Odd or Even?
	LD A,90			;Beacon right if even
	JR Z +1			;Jump if even
	INC A			;Beacon left if odd
	LD (Roof+4),A		;Set Beacon chr
	LD A,B			;Restore random number
	AND 07			;Range 0 to 7
	ADD A,04		;Range 4 to 11, building chr
	LD (Floor+3),A		;Set building chr
	LD E,0C			;Upper limit=12
	CALL Modulus		;A=0 to 12,
	INC A			;A=1 to 13=Building height
	LD B,A			;B=building height
	LD A,17			;23 bytes/column
	SUB B			;A-B=Sky value
	LD HL,(Vars)		;PRINT AT 23,31
	DEC HL
	DEC HL
	LD (HL),89		;Print sidewalk
	LD DE,0021		;33 bytes/row
@Floor
	SBC HL,DE		;Next floor up
	LD (HL),00		;Print "POKEd" floor chr
	DJNZ Floor		;Print B floors
	LD B,A			;B=Sky count
	CP 0A			;Print beacon if A=10
	JR Z Roof		;Print roof chr
	CP 0B			;Print blinker if A=11
	JR NZ +4
	LD A,81			;Blinker chr
	JR +6
	CP 0C			;Print flasher if A=12
	JR NZ +10
	LD A,99			;Flasher chr
	LD (Roof+4),A		;Set rooftop chr
@Roof
	DEC B			;Decrease sky count
	SBC HL,DE		;Roof print position
	LD (HL),00		;Print "POKEd" rooftop chr
;Sky
	LD A,80			;Sky chr
;SkyLoop
	SBC HL,DE		;Sky print position
	LD (HL),A		;Print sky
	DJNZ -5			;Print B sky chrs
@RoofTop
	LD HL,(DFile)
	LD BC,0319
	LD A,(HL)
	CP 80			;Skip these chrs
	JR Z RoofPrnt+1
	CP 76
	JR Z RoofPrnt+1
	BIT 7,A
	JR Z RoofPrnt+1
;Beacon animation
	LD DE,91B4
	CP D
	JR Z RoofPrnt
	LD DE,B490
	CP D
	JR Z RoofPrnt
	LD DE,9091
	CP D
	JR Z RoofPrnt
;Blinker animation
	LD DE,8182
	CP D
	JR Z RoofPrnt
	INC D
	DEC E
	CP D
	JR Z RoofPrnt
;Flasher animation
	LD DE,999A
	CP D
	JR Z RoofPrnt
	INC D
	DEC E
	CP D
	JR NZ +1
@RoofPrnt
	LD (HL),E
	INC HL
	DEC BC 
	LD A,B 
	OR C 
	JR NZ RoofTop+6
	CALL Delay		;Delay
	CALL KeySort		;Test for keypress
	JP Cityscape		;Build next building
;==============================
; Vertigo
;==============================
@Vertigo
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	LD B,A			;Save A
	AND 01			;Odd or Even?
	JR Z +3			;Yes, move on
	XOR A			;Closed spiral
	JR +2
	LD A,80			;Open spiral
	LD (Style+1),A		;Set spiral style
	LD A,B			;Restore A
	AND 3F			;Range 0 to 63
	INC A			;Range 1 to 64
	LD (IY+09),A		;Set gen counter
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	AND 0F			;Range 0 to 15
	LD B,00
	LD C,A			;BC=Chr number
	LD HL,Pixels		;Pixel chrs
	ADD HL,BC		;HL=chr address
	LD A,(HL)		;Get chr
	LD (NxtSpiral+1),A	;Set chr to print
	CALL Spiral		;Print spiral
	CALL KeySort		;Test keypress
	DEC (IY+09)		;Decrease gen counter
	JR NZ -11		;Loop until gen count=0
	JR Vertigo		;Restart with new parameters
;================================
; Print open or closed spiral
;================================
@Spiral
	LD HL,(DFile)		;Get DFile
	LD C,17		 	;Rows to clear, 21 
	LD B,20 		;Columns to clear, 32
@NxtSpiral
	LD A,00			;Chr to print
	LD D,00			;0=BonW, 80=WonB 
	ADD A,D			;Set as BonW or WonB chr
	LD D,B 			;D=column counter
;Top
	INC HL 
	LD (HL),A 
	CALL Pause
	DEC D 
	JR NZ -8
;Right
	DEC B 
	LD E,C 
	PUSH BC 
	LD BC,0021 
	ADD HL,BC 
	LD (HL),A 
	CALL Pause
	DEC E 
	JR NZ -8
;Bottom
	POP BC 
	DEC C 
	LD D,B 
	DEC HL 
	LD (HL),A 
	CALL Pause
	DEC D 
	JR NZ -8
;Left
	LD E,A 
	LD A,B 
	CP 09 
	RET Z 
	LD A,E 
	DEC B 
	LD E,C 
	PUSH BC 
	LD BC,0021 
	SBC HL,BC 
	LD (HL),A 
	CALL Pause
	DEC E 
	JR NZ -9
@Style
	ADD A,80		;True spiral instead of rectangle
	LD (HL),A 
	POP BC 
	DEC C 
	LD A,(NxtSpiral+3)	;Change BonW<->WonB
	ADD A,80
	LD (NxtSpiral+3),A
	JR NxtSpiral
@Pause
	PUSH AF 
	LD A,(Delay+4) 		;Get delay value
	AND A
	JR Z +3
	DEC A 
	JR NZ -3
	POP AF 
	RET
;================================
; Dancers
;================================
@Dancers
	CALL Release		;Wait for key release
	LD HL,(DFile)		;Get address of line 23
	LD BC,02F8		;Offset to line 23
	ADD HL,BC
	LD (DancerX+2),HL	;Set line 23 address
	CALL CLSBW		;CLS with black or white bkg
	LD A,D			;A=BKG chr
	LD (SetBW+1),A		;Set BonW or WonB value
	LD (ClearPit+2),A	;Set erase chr
	LD BC,DanceDat		;Dancers count addr
	LD DE,Dancerz		;Dancers table
@DancerData
	PUSH DE			;PUSH chr table addr
	CALL RND8Bit		;Get random number
	AND 1F			;Range=0 to 31
	INC C			;DancerX Xcoord addr
	LD (BC),A		;Set DancerX Xcoord
	INC C			;DancerX chr
	POP DE			;POP chr table addr
	LD A,(DE)		;Get DancerX chr
	LD (BC),A		;Set DancerX chr
	INC DE			;Increase table pointer
	LD A,5C			;Test for end of data section
	CP C			;End of data?
	JR NZ DancerData	;No, loop back
	LD A,01			;Start with 1 dancer
@DancePars
	LD (DanceDat),A		;Set dancer count
	LD (IY+09),FF		;Set scroll counter to 255=10.6 screens
@DanceLoop
	LD HL,DanceDat		;Start of dancers data
	LD B,(HL)		;Get number of Dancers
@MoshPit
	INC HL			;DancerX Xcoord
	LD D,00			;Set Ycoord
	LD E,(HL)		;Set Xcoord	
	INC HL			;DancerX chr
	LD A,(HL)		;Get DancerX Chr
@SetBW
	ADD A,00		;00 or 128, POKEd value
	CALL DancerX		;PRINT AT 23,E;DancerX
	DJNZ MoshPit		;Loop until B=0
	CALL ScrollUp
	LD HL,DanceDat		;Clear the dance floor
	LD B,(HL)		;Get dancers count
@ClearPit
	INC HL			;DancerX Xcoord
	LD A,00			;00 or 128, POKEd value
	LD D,00			;Set Ycoord
	LD E,(HL)		;Set Xcoord
	CALL DancerX		;PRINT AT 23,E;A
	PUSH HL			;PUSH DancerX addr
	CALL RND8Bit		;Get random number
	POP HL			;POP DancerX addr
	AND 01			;Odd or even?
	LD A,(HL)		;A=Xcoord
	JR Z +9			;Jump if even
	INC A			;Step right
	CP +32			;Right edge?
	JR C SetStep
;	XOR A			;Wrap left or...
	DEC A			;Bounce off wall
	DEC A
	JR SetStep
	DEC A			;Step left
	CP FF
        JR NZ SetStep
;	LD A,+31		;Wrap right or...
	INC A			;Bounce off wall
	INC A
@SetStep
	LD (HL),A		;Set new Xcoord
	INC HL			;DancerX chr
	DJNZ ClearPit
	CALL Keysort		;Sort out any key presses
	CALL Delay
	DEC (IY+09)		;Decrease gen counter
	JR NZ DanceLoop		;Loop until gen counter=0
	LD A,(DanceDat)		;Get dancers count
@IncOrDec
	INC A			;Dancers=Dancers+/-1
	CP 0B			;9?
	JR Z +9
	AND A			;0?
	JR NZ DancePars		;No, loop with increased dancers
	LD A,3C			;INC A instruction
	LD C,02
	JR +4
	LD A,3D			;DEC A instruction
	LD C,09
	LD (IncOrDec),A
	LD A,C
	JP DancePars
;================================
;PRINT AT 23,E;A
;================================
@DancerX
	PUSH HL
	LD HL,0000		;DFile line 23 addr, POKEd from above
	ADD HL,DE		;Add Xcoord offset
	LD (HL),A		;Print A
	POP HL
	RET
;================================
; Scroll Up
;================================
@ScrollUp
	LD HL,(DFile)
	PUSH HL 
	LD DE,0021 
	ADD HL,DE 
	POP DE 
	LD BC,02F7 
	LDIR
	RET
;================================
; Setup DFile pointers
;================================
@SetupDF
	LD HL,(DFile)		;Get DFile
	LD (ZXDFile+1),HL	;Set ZXDFile
	LD HL,(altDFile)	;altDFile addr
	LD (WorkDF),HL		;Start with altDFile
	RET
;================================
; Get random cell chr
;================================
@PikCell
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	AND 0F			;Range 0 to 15
	LD B,00
	LD C,A			;BC=Chr number
	LD HL,CellChrs		;Cell chrs list
	ADD HL,BC		;HL=chr address
	LD A,(HL)		;Get chr
	ADD A,(IY+61)		;Add BKG
	CP (IY+60)		;Loop back if same as last cell
	JR Z PikCell
	CP (IY+61)		;Loop back if same as BKG
	JR Z PikCell
	LD (Cell),A		;Set chr
	RET
;================================
; Set print chr to (BKG) or (Cell)
; based on random number generated.
;================================
@PrintCell
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	LD L,A			;L=8-Bit random number, range 0 to 255
	LD A,(Density)		;Get density value
	CP L			;L>Density?
	LD A,(Cell)		;Get chr to print
	JR C +3			;Yes, print cell
	LD A,(BKG)		;No, print bkg
	LD (PokeA+1),A		;POKE chr to print
	RET
;================================
; Print A to the four quadrants
; of the screen.
;================================
@PrintQuads
	CALL PrintX		;Print to NW quadrant B,C
	LD A,1F
        SUB C
        LD C,A
	CALL PrintX		;Print to NE quadrant B,31-C
	LD A,17
        SUB B
        LD B,A
	CALL PrintX		;Print to SE quadrant 25-B,31-C
	LD A,1F			
        SUB C
	LD C,A
;================================
; Locate print position in DFile
; then print POKEd chr.
;================================
@PrintX
	PUSH DE
	PUSH BC
	INC B			;Ycoord must be > 0
	LD HL,(DFile)		;Get start of DFile
	INC HL
	LD DE,0000		;Initialize DE
	ADD HL,DE		
	LD E,21			;DE=33 bytes/row
	DJNZ -5			;Add Bx33 to start of DFile
	LD E,C			;DE=Xcoord offset
	ADD HL,DE		;Add X offset to DE
@PokeA
	LD (HL),00		;POKEd chr to print
	POP BC			;POP Y/Xcoords
	POP DE
	RET
;================================
; Delay routine
; All registers preserved
; HL' not change
;================================
@Delay
	EXX				;Save main reg's
	PUSH AF				;Save A reg
	LD BC,1001	 		;Delay Loop
	DEC BC 
	LD A,B 
	OR C 
	JR NZ -5
	POP AF				;Restore A reg
	EXX				;Restore main reg's
	RET
;================================
; Sort out the key presses
;================================
@KeySort
	CALL Delay
	LD HL,LoopFlag		;Loopflag addr
	BIT 7,(HL)		;Test stopped bit.
	JR Z KeyTest		;If 0 move on, still playing
	RES 7,(HL)		;Tune ended, reset stopped flag
	BIT 0,(HL)		;Test loop flag
	JR NZ TuneDone		;If NZ end video
@KeyTest
	CALL Keyscan
	LD HL,Delay+4		;Delay count Hi byte
;KeyS
	CP "S"			;"S"lower?
	JR NZ +2		;No, move on
	INC (HL)		;Increase delay count
	RET
;KeyF
	CP "F"			;"F"aster?
	JR NZ +6		;No, move on
	LD A,(HL)		;Test for 0
	AND A
	JR Z +1
	DEC (HL)		;Decrease delay count
	RET
;KeyI
	CP "I"			;"I"reg cycle?
	JR NZ +16		;No, move
@IReg
	LD A,FE			;Current Ireg value
	INC A			;Add 2
	INC A
	CP 20			;32?
	JR NZ +1		;No, move on
	XOR A			;Yes, reset to 0
	LD (Ireg+1),A		;Store Ireg
	LD I,A			;Set Ireg
	JR Release		;Pause the display until key release	
;KeyQ
	CP "Q"			;"Q"uit?
	JR NZ Release		;Pause the display if key pressed
@TuneDone
	CALL 49A5		;Restore screen
	POP BC			;Clear the stack
	LD A,1E			;Reset IReg just in case
	LD I,A
	CALL Release		;Wait for key release
	LD BC,+90		;
	RET			;GOTO 90
;================================
; Test for keypress.
; On exit...
;  A=key chr code
;================================
@KeyScan
	CALL Keyboard		;Call keyboard scanning routine
	LD B,H 			;BC=Raw Key values
	LD C,L 
	LD D,C			;Test for keypress 
	INC D 
	LD A,01 
	RET Z 			;No keypress
	CALL Decode		;Call keyboard decode routine, BC to ZX81 codes
	LD A,(HL)		;A=CHR CODE
	RET
;================================
; Wait for key press
;================================
@Wait
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR Z Wait		;No, loop back
	RET
;================================
; Wait for key release
;================================
@Release
	LD A,(LastK)		;Test for keypress
	CP FF			;Keypressed?
	JR NZ Release		;Yes, loop back
	RET
;================================
; Clear screen with random " " or "g "
;================================
@CLSBW
	CALL Rnd8Bit		;Get 8bit random number, 0 TO 255
	AND 01
	JR Z +2			;0, move on
	LD A,80			;1, "g "
;================================
; Clear screen with chr A
;================================
@CLS
	LD (BKG),A		;Set BKG chr
	LD D,A			;CLS with BKG chr
	LD HL,(DFile)
	LD BC,0318		;Byte count
	INC HL
	BIT 6,(HL)		;N/L?
	JR NZ +1		;Yes, skip it
	LD (HL),D		;Print BKG chr.
	DEC BC 			;Decrease counter
	LD A,B 
	OR C 
	JR NZ -11		;Loop until BC=0
	RET
;================================
; 8-Bit Random Number Generator
; Reg BC not used
; On exit...
;  Reg A=random number in the
;        range 0 to 255
;================================
@Rnd8Bit
	LD DE,0000		;Seed
	LD HL,(Frames)     	;Create pointer to ROM address
	DEC DE	
	ADD HL,DE
	LD A,H
	AND 1F
	LD H,A			;Limit address to <=8192
	LD (Rnd8Bit+1),HL	;New seed
	LD A,(HL)		;Get value at ROM address
	RET
;================================
; Modulus code to ensure 
; number in desired range.
; On entry...
;  E=Upper Limit
; Reg BC not used
; On exit...
;  A=random number between 0 and
;    upper limit
;================================
@Modulus
	LD D,00
	INC E			;DE=dividend
	PUSH DE
	CALL Rnd8Bit		;Get 8-Bit random number
	POP DE
	LD H,D			
	LD L,A			;HL=8-Bit random number
@ModLoop
	OR A			;Clear CY
	SBC HL,DE		;Subtract dividend from HL
	JP P ModLoop		;Keep doing it till negative
	ADD HL,DE		;Add the divided back again
	LD A,L			;Return results in A
	DEC E
	RET
;================================
; Data tables
;================================
@CellChrs			;Cell chr list
	01 03 05 06 07 08 09 0B 12 17 1B 34 3B 3D 80
@MazeChr			;Endless maze chrs
	80 01 02 87 04 05 83 03 85 81 82 07 84 06 86 00
@Pixels				;Vertigo chr list
        00 01 03 05 06 06 08 09 0A 80 81 83 85 03 05 06
@Dancerz			;Dancers
	01 04 0A 83 17 15 34 1C 88 80
@Felt				;Dice background
	00 08 80 88 8E 97 08
@Line1				;Die line 1
	07 03 84
@Line2				;Die lines 2 to 11
	05 00 85
@Line12				;Die line 12
	82 83 81
